
/* 
 * File:   GSLData.h
 * Author: skidmarks
 *
 * Created on December 15, 2017, 2:30 PM
 */

#ifndef GSLDATA_H
#define GSLDATA_H

# include <gsl/gsl_errno.h>

# include <sstream>
# include <string>

using namespace std;
namespace gsl {
   
# define reason(str) str.c_chr
   
# define GSLvoidError(reason, gsl_errno)           { gsl_error (reason.c_str(), __FILE__, __LINE__, gsl_errno); }
   
# define GSLObjError(reason, gsl_errno, retValue) { GLSvoidError(reason, gsl_errno);  return retValue; }

class GSLData {
public:
   enum gsl { gslDummy = 0
            , gslBlock = 1
            , gslMatrix
            , gslMatrixView
            , gslVector
            , gslVectorView
   }; // gsl
   
protected:
   const void*    structPtr;
   const size_t   totalSize;
   const gsl      ident;
private:
   GSLData(const GSLData& orig) : structPtr(orig.block()), totalSize(orig.totalSize), ident(orig.ident) {}
   
public:
   GSLData(const size_t size, const gsl ident, const void* ptr) : structPtr(ptr), totalSize(size), ident(ident) {};
   virtual ~GSLData() { };
   virtual string toString() = 0;
      
   /******************************************************
    *             Reading and Writing Blocks             *
    ******************************************************/
   
   /******************************************************
    *                Universal  Properties               *
    ******************************************************/
   
   virtual void*  data()     = 0;               // pointer to gsl_block
   
   inline gsl     id() const    { return ident; };       // block identifier
   inline size_t  size() const  { return totalSize; }    // number of elements in block
   inline void*   block() const { return const_cast<void*>(structPtr); } // pointer to gsl_block
private:

};
} // namespace gls
#endif /* GSLDATA_H */

