
/* 
 * File:   GSLMatrix.cpp
 * Author: skidmarks
 * 
 * Created on December 15, 2017, 2:54 PM
 */

# include <gsl/gsl_matrix_double.h>

#include "GSLMatrix.h"

using namespace gsl;

GSLMatrix::GSLMatrix(size_t row, size_t col)             : GSLData(row*col, gslMatrix, (gsl_matrix_alloc(row, col))) { };
GSLMatrix::GSLMatrix(size_t row, size_t col, int calloc) : GSLData(row*col, gslMatrix, (gsl_matrix_calloc(row, col))) { };
GSLMatrix::GSLMatrix(_gsl_matrix_view matrix)            : GSLData( matrix.matrix.size1 * matrix.matrix.size2
                                                         , gslMatrixView
                                                         , &matrix.matrix) { };
GSLMatrix::GSLMatrix(_gsl_matrix_const_view matrix)      : GSLData( matrix.matrix.size1 * matrix.matrix.size2
                                                         , gslMatrixView
                                                         , &matrix.matrix) { };
GSLMatrix::GSLMatrix(const GSLMatrix& orig)              : GSLData(orig.size(), orig.id(), orig.block()) {};

GSLMatrix::~GSLMatrix() {
   if (id() == gslMatrix) gsl_matrix_free((gsl_matrix*)block()); }

   /*******************************************************
    *******************************************************/


/*
   int= GSL_EFAILED
   gcc -DHAVE_INLINE
   GSL_RANGE_CHECK_OFF
   GSL_COO_INTLINE
   gsl_check_range
*/

GSLMatrix   GSLMatrix::submatrix(size_t row, size_t col, size_t rows, size_t cols) {
   return GSLMatrix(gsl_matrix_submatrix((gsl_matrix*)block(), row, col, rows, cols));
}; // 

GSLMatrix   GSLMatrix::view_vector(GSLVector& v, size_t rows, size_t cols) {
   return GSLMatrix(gsl_matrix_view_vector((gsl_vector*)v.block(), rows, cols));
}; // 

GSLVector GSLMatrix::view_array_with_tda(double * base, size_t rows, size_t cols, size_t tda) {
   return GSLVector(view_array_with_tda(base, rows, cols, tda));
}; // GSLVector GSLMatrix::view_array_with_tda(double * v, size_t rows, size_t cols, size_t tda)

GSLMatrix   GSLMatrix::view_vector_with_tda(GSLVector& v, size_t rows, size_t cols, size_t tda) {
   return GSLMatrix(gsl_matrix_view_vector_with_tda((gsl_vector*)v.block(), rows, cols, tda));
}; // 


GSLVector   GSLMatrix::subrow(size_t row, size_t offset, size_t number) {
   return GSLVector(gsl_matrix_subrow((gsl_matrix*)block(), row, offset, number));
}; // 

GSLVector   GSLMatrix::subcol(size_t col, size_t offset, size_t number) {
   return GSLVector(gsl_matrix_subrow((gsl_matrix*)block(), col, offset, number));
}; // 

const GSLMatrix   GSLMatrix::const_submatrix(size_t row, size_t col, size_t rows, size_t cols) {
   return GSLMatrix(gsl_matrix_const_submatrix((gsl_matrix*)block(), row, col, rows, cols));
}; // 

const GSLMatrix   GSLMatrix::const_view_array_with_tda(double * base, size_t rows, size_t cols, size_t tda) {
   return GSLMatrix(gsl_matrix_const_view_array_with_tda(base, rows, cols, tda));
}; // GSLVector GSLMatrix::const_view_array_with_tda(double * base, size_t rows, size_t cols, size_t tda)

const GSLMatrix   GSLMatrix::const_view_vector(GSLVector& v, size_t rows, size_t cols) {
   return GSLMatrix(gsl_matrix_const_view_vector((gsl_vector*)v.block(), rows, cols));
}; // 

const GSLMatrix   GSLMatrix::const_view_vector_with_tda(GSLVector& v, size_t rows, size_t cols, size_t tda) {
   return GSLMatrix(gsl_matrix_const_view_vector_with_tda((gsl_vector*)v.block(), rows, cols, tda));
}; // 

