
/* 
 * File:   GSLVector.cpp
 * Author: skidmarks
 * 
 * Created on December 15, 2017, 2:54 PM
 */

# include <gsl/gsl_vector.h>
# include <gsl/gsl_vector_complex_double.h>

# include "GSLVector.h"

using namespace std;
using namespace gsl;

GSLVector::GSLVector(size_t size) : GSLData(size, gslVector, (gsl_vector_alloc(size))) { };
GSLVector::GSLVector(size_t size, int calloc) : GSLData(size, gslVector, gsl_vector_calloc(size)) { };
GSLVector::GSLVector(_gsl_vector_view vector) : GSLData( vector.vector.size
                                                       , gslVectorView
                                                       , &vector.vector) { };
GSLVector::GSLVector(_gsl_vector_const_view vector) : GSLData( vector.vector.size
                                                             , gslVectorView
                                                             , &vector.vector) { };

GSLVector::GSLVector(const GSLVector& orig) : GSLData(orig.size(), orig.id(), orig.block()) { };

GSLVector::~GSLVector() { if (id() == gslVector) gsl_vector_free((gsl_vector*)block()); }

/**********************************************************
 *********************************************************/
 
// int= GSL_EFAILED
 
// gcc -DHAVE_INLINE
// GSL_RANGE_CHECK_OFF
// GSL_COO_INTLINE
// gsl_check_range

GSLVector GSLVector::subvector(const size_t offset, const size_t number) {
   return GSLVector(gsl_vector_subvector((gsl_vector *)block(), offset, number));
}; // GSLVector GSLVector::subvector( size_t offset, size_t n)

const GSLVector GSLVector::const_subvector(size_t offset, size_t number) {
   return GSLVector(gsl_vector_const_subvector((gsl_vector *)block(), offset, number));
}; // const GSLVector GSLVector::const_subvector(size_t offset, size_t n)

GSLVector GSLVector::subvector_with_stride(const size_t offset, const size_t number, const size_t stride) {
   return GSLVector(gsl_vector_subvector_with_stride((gsl_vector *)block(), offset, stride, number));   
}; //GSLVector  GSLVector::subvector_with_stride(size_t offset, size_t sgride, size_t n)

const GSLVector GSLVector::const_subvector_with_stride(const size_t offset, const size_t number, const size_t stride) {
   return GSLVector(gsl_vector_const_subvector_with_stride((gsl_vector *)block(), offset, stride, number));  
}; // GSLVector GSLVector::const_subvector_with_stride(size_t offset, size_t stride, size_t n)

GSLVector GSLVector::view_array(double* base, const size_t number, size_t col) {
   return GSLVector(gsl_vector_view_array(base, number));
}; // GSLVector::view_array(double * base, size_t n)

const GSLVector GSLVector::const_view_array(const double* base, const size_t number, size_t col) {
   return GSLVector(gsl_vector_const_view_array(base, number));
}; // GSLVector::const_view_array(const double * base, size_t n) 

GSLVector GSLVector::view_array_with_stride(double * base, size_t stride, size_t size, size_t col) {
   return GSLVector(gsl_vector_view_array_with_stride(base, stride, size));
}; // GSLVector::view_array_with_stride(double * base, size_t stride, size_t size)

const GSLVector GSLVector::const_view_array_with_stride(const double * base, size_t stride, size_t size) {
   return GSLVector(gsl_vector_const_view_array_with_stride(base, stride, size));
}; // GSLVector GSLVector::const_view_array_with_stride(const double * base,size_t stride, size_t n)
