
/* 
 * File:   GSLVector.h
 * Author: skidmarks
 *
 * Created on December 15, 2017, 2:54 PM
 */

#ifndef GSLVECTOR_H
#define GSLVECTOR_H

# include <gsl/gsl_vector.h>
# include <string>

# include "GSLData.h"

using namespace std;
namespace gsl {
class GSLVector  : public GSLData {
private:
  const string errMsg = "function not implemented in GSLVector";
   
public:
   GSLVector(size_t size);
   GSLVector(size_t size, int calloc);
   GSLVector(_gsl_vector_view vector);
   GSLVector(_gsl_vector_const_view vector);
   GSLVector(const GSLVector& orig);

   virtual ~GSLVector();
   virtual string toString() { return ""; };
   
   /******************************************************
    *        Common Matrix and Vector properties         *
    ******************************************************/
   virtual void*     data()                     { return ((gsl_vector*)block())->data;}
   virtual double    gslMax()                   { return gsl_vector_max((gsl_vector*)block());};
   virtual double    gslMin()                   { return gsl_vector_min((gsl_vector*)block());};
   
   virtual int       isnull()                   { return gsl_vector_isnull((gsl_vector*)block());};
   virtual int       isneg()                    { return gsl_vector_isneg((gsl_vector*)block());};
   virtual int       isnonneg()                 { return gsl_vector_isnonneg((gsl_vector*)block());};
   virtual int       ispos()                    { return gsl_vector_ispos((gsl_vector*)block());};
      
   /******************************************************
    *    Accessing & initializing Matrices and Vectors   *
    ******************************************************/
   virtual const double * const_ptr(size_t row, size_t col = 0)  { return gsl_vector_ptr((gsl_vector*)block(), row);};
   virtual double    get(const size_t row, const size_t col = 0) { return gsl_vector_get((gsl_vector*)block(), row);};
   virtual double*   ptr( size_t row, size_t col = 0)            { return gsl_vector_ptr((gsl_vector*)block(), row);};
   virtual void      set(size_t row, double value)               { gsl_vector_set((gsl_vector*)block(), row, value);};
   virtual void      set_zero()                 { gsl_vector_set_zero((gsl_vector*)block());};
   virtual void      set_all(double x)          { gsl_vector_set_all((gsl_vector*)block(), x);};
   virtual void      set(const double x, const size_t row, const size_t col = 0)
                                                { return gsl_vector_set((gsl_vector*)block(), row, x);};
   /******************************************************
    *            Matrix and Vector Arithmetic            *
    ******************************************************/
   virtual int add(GSLVector& b)             { return gsl_vector_add((gsl_vector*)block(), (gsl_vector*)b.block());};
   virtual int add_constant(const double x)  { return gsl_vector_add_constant((gsl_vector*)block(),x);};
   virtual int sub(GSLVector& b)             { return gsl_vector_sub((gsl_vector*)block(), (gsl_vector*)b.block());};
   virtual int scale(double x)               { return gsl_vector_scale((gsl_vector*)block(),x);};
      
   /******************************************************
    *            Matrix and Vector Operations            *
    ******************************************************/
   virtual int    equal(GSLVector& b)        { return gsl_vector_equal((gsl_vector*)block(), (gsl_vector*)b.block());};
   virtual size_t max_index()                { return gsl_vector_max_index((gsl_vector*)block());};
   virtual size_t min_index()                { return gsl_vector_min_index((gsl_vector*)block());};
   virtual void   minmax(double * min, double * max) 
                                             { gsl_vector_minmax((gsl_vector*)block(), min, max);};
   virtual void   minmax_index(size_t * min, size_t * max) { gsl_vector_minmax_index((gsl_vector*)block(), min, max);};
   
   /******************************************************
    *            Copying Matrices and Vectors            *
    ******************************************************/
   virtual int memcpy(GSLVector& dest)      { return gsl_vector_memcpy((gsl_vector*)dest.block(), (gsl_vector*)block()); };
   virtual int swap(GSLVector& w)           { return gsl_vector_swap((gsl_vector*)block(), (gsl_vector*)w.structPtr);};
   
   /******************************************************
    *                  Vector Arithmetic                 *
    ******************************************************/
   virtual int div(GSLVector& b)            { return gsl_vector_div((gsl_vector*)block(), (gsl_vector*)b.block());};
   virtual int mul(GSLVector& b)            { return gsl_vector_mul((gsl_vector*)block(), (gsl_vector*)b.block());};
   
   /******************************************************
    *                 Common Views                       *
    ******************************************************/
   virtual GSLVector view_array(double* base, const size_t number, size_t col=0);
   virtual const GSLVector const_view_array(const double* base, const size_t number, size_t col=0);
   virtual GSLVector view_array_with_stride(double * base, size_t stride, size_t size, size_t col = 0);
   virtual const GSLVector const_view_array_with_stride(const double * base, size_t stride, size_t size);

   /******************************************************
    *                    Vector Views                    *
    ******************************************************/
   virtual GSLVector subvector(const size_t offset, const size_t number);
   virtual GSLVector subvector_with_stride(const size_t offset, const size_t number, const size_t stride);
   
   /******************************************************
    *                const Vector Views                  *
    ******************************************************/
   virtual const GSLVector const_subvector(const size_t offset, const size_t number);
   virtual const GSLVector const_subvector_with_stride(const size_t offset, const size_t number, const size_t stride);
   
   /******************************************************
    *                    Vector Misc                     *
    ******************************************************/
   virtual int swap_elements(size_t i, size_t j)  { return gsl_vector_swap_elements((gsl_vector*)block(), i, j);};
   virtual int reverse()                          { return gsl_vector_reverse((gsl_vector*)block());};

private:

}; // class GSLVector
} // namespace gls
#endif /* GSLVECTOR_H */

